# CVS ID : $Id: ex2.tcl,v 1.4 2011/07/09 15:15:01 wuthrich Exp $
# --------------------------------------------------
#
# Example of an experiment consisiting of a
# potential step followed by a cyclic voltammetry
#
# Demonstrates the utilisation of notes
#
# (c) Rolf Wuthrich
#     2010 Concordia University
#
# author:  Rolf Wuthrich
# email:   wuthrich@encs.concordia.ca
# version: v1 ($Revision: 1.4 $)
#
# --------------------------------------------------

# clears memory
clear
notes -clear

# defines the name of the experiment
set ::ectk::expName Nickel

# adds a note
set d 1.5
set h 2.0
notes -add {Working electrode:}
notes -add { - Nickel wire}
notes -add { - Diameter = $::d mm}
notes -add { - Height   = $::h mm}
notes -add { - Surface  = [expr {$::h*3.14*$::d}] mm^2}
notes -add {}
notes -add {Counter electrode:}
notes -add { - Platinum wire}
notes -add {}
notes -add {Supporting electrolyte:}
notes -add { - NaOH 1M}

# Switch on the cell
cell -on

# Defines a cyclic voltammetry experiment
cv -eInitial 0.5 \
   -eFinal 0.8 \
   -scanRate 0.10 \
   -nCycles 2 \
   -preProcess {set E 0.5}

# Defines a chronoamperometry experiment with a trigger
trigger -action {terminate $myca} \
        -triggerScript {if {$t>2} {return STOP} else {return CONTINUE}} \
        -verbose

set myca [ca -eInitial -1.2 -eFinal 1.8 -stepTime 1.0 -stepDuration 10]

# Switch off the cell
cell -off
