// CVS ID : $Id: ECApp.cc,v 1.12 2011/07/09 15:15:01 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECApp
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.12 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// 08.06.2010 Updated for ectk dir structure   R. Wuthrich
// 20.06.2010 Added ECRec                      R. Wuthrich
// 20.06.2010 Added findECProcedure            R. Wuthrich
// ---------------------------------------------------------------

#include <cstring>
#include <tcl.h>
#include <iostream>
#include <tclreadline.h>
#include "ECApp.h"
#include "ECEvent.h"
#include "ECProcess.h"
#include "ECRec.h"
#include "ECIO.h"
#include "ECCell.h"
#include "ECTrigger.h"
#include "ECCV.h"
#include "ECCA.h"
#include "ECStep.h"
#include "ECPulse.h"
#include "ECUserProc.h"

using namespace std;

// ------------------------------------------------------------
//
// ECTK license
//

static CONST char* ectk_license =
"   Copyright (c) 2010, Rolf Wuthrich <wuthrich@encs.concordia.ca>\n"
"   All rights reserved.\n "
"   \n"
"   Redistribution and use in source and binary forms, with or without\n"
"   modification, are permitted provided that the following conditions\n"
"   are met:\n"
"   \n"
"     * Redistributions of source code must retain the above copyright\n"
"       notice, this list of conditions and the following disclaimer.\n"
"     * Redistributions in binary form must reproduce the above copyright\n"
"       notice, this list of conditions and the following disclaimer in the\n"
"       documentation and/or other materials provided with the distribution.\n"
"     * Neither the name of Rolf Wuthrich nor the names of contributors\n"
"       to this software may be used to endorse or promote products derived\n"
"       from this software without specific prior written permission.\n"
"       \n"
"   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS\n"
"   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT\n"
"   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR\n"
"   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR\n"
"   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,\n"
"   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,\n"
"   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR\n"
"   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF\n"
"   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING\n"
"   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS\n"
"   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.";


// ------------------------------------------------------------
//
// New Tcl commands defined by ECApp
//


// ---- Tcl ps command
//

int ECTk_PsCmd(ClientData clientData, Tcl_Interp *interp, 
	       int objc,Tcl_Obj *const objv[])
{
  ECApp*     app=(ECApp *)clientData;
  ECProcess* p=app->getFirstECProcess();

  if (objc!=1) 
    {
      Tcl_WrongNumArgs(interp,1,objv,NULL);
      return TCL_ERROR;
    }

  char str[100];
  char user_name[15];

  Tcl_AppendResult(interp,
		   "OWNER     PID  PROCESS NAME         CMD        STATUS\n",
		   NULL);
  while (p!=NULL)
    {
      sprintf(str,"::ectk::%s",p->getProcessOwner());
      if (Tcl_GetVar(interp,str,0)!=NULL)
	strcpy(user_name,Tcl_GetVar(interp,str,0));
      else
	strcpy(user_name,p->getProcessOwner());

      sprintf(str,"%-9s %-3s  %-20s %-10s %s\n",
	      user_name,
	      p->getID(),
	      p->getECObjName(),
	      Tcl_GetVar2(interp,"::ectk::procList",p->getID(),0),
	      p->getStatusStr());
      Tcl_AppendResult(interp,str,NULL);
      p=p->getNextECProcess();
    }
  return TCL_OK;
}


// ---- Tcl lrec command
//

int ECTk_lrecCmd(ClientData clientData, Tcl_Interp *interp, 
		 int objc,Tcl_Obj *const objv[])
{
  ECApp* app=(ECApp *)clientData;
  ECRec* p=app->getFirstECRecord();

  if (objc!=1)
    {
      Tcl_WrongNumArgs(interp,1,objv,NULL);
      return TCL_ERROR;
    }

  char str[100];

  Tcl_AppendResult(interp,"ECRECORD     PID   # OF RECORDS\n",NULL);
  while (p!=NULL)
    {
      sprintf(str,"%-12s %-3s   %-3d\n",
	      Tcl_GetVar2(interp,"::ectk::recList",p->getID(),0),
	      p->getID(),
	      p->getNbrECRecords());
      Tcl_AppendResult(interp,str,NULL);
      p=p->getNextECRec();
    }
  return TCL_OK;
}


// ---- Tcl start command
//

int ECTk_StartCmd(ClientData clientData, Tcl_Interp *interp, 
		  int objc,Tcl_Obj *const objv[])
{
  ECApp* app=(ECApp *)clientData;

  if (objc>2)
    {
      Tcl_WrongNumArgs(interp,1,objv,"?cellID?");
      return TCL_ERROR;
    }

  if (app->getEvent()->cellStatus()==CellOn)
    {
      Tcl_AppendResult(interp,
		       "can't start experiment; cell is already on",NULL);
      return TCL_ERROR;
    }

  ECProcess* p;
  if (objc==1)
    p=app->findECProcess("1");
  else
    p=app->findECProcess(Tcl_GetString(objv[1]));

  if (p==NULL)
    {
      Tcl_AppendResult(interp,"invalid cell PID",NULL);
      return TCL_ERROR;     
    }

  // loops over all processes in order to reset them in Waiting status
  while (p!=NULL)
    {
      if (p->getStatus()==Terminated)
	p->setStatus(Waiting);
      p=p->getNextECProcess();
    }

  app->getEvent()->setCmdEvent(CmdStart,NULL);

  Tcl_Eval(interp,"cons \"+---------------------------------------+\"");
  Tcl_Eval(interp,"cons \"|  Starting electrochemical experiment  |\"");
  Tcl_Eval(interp,"cons \"+---------------------------------------+\"");
  Tcl_Eval(interp,"cons \"|     [exec date]      |\"");
  Tcl_Eval(interp,"cons \"+---------------------------------------+\"");
  return TCL_OK;
}


// ---- Tcl stop command
//

int ECTk_StopCmd(ClientData clientData, Tcl_Interp *interp, 
		 int objc,Tcl_Obj *const objv[])
{
  ECApp* app=(ECApp *)clientData;

  if (objc!=1)
    {
      Tcl_WrongNumArgs(interp,1,objv,NULL);
      return TCL_ERROR;
    }

  ECEvent* ev=app->getEvent();
  ev->setCellStatus(CellOff);
  ev->setCmdEvent(CmdStop,NULL);
  return TCL_OK;
}


// ---- Tcl activate command
//
//      Only intended for debugging not for actual use
//

int ECTk_ActivateCmd(ClientData clientData, Tcl_Interp *interp, 
		     int objc,Tcl_Obj *const objv[])
{
  ECApp* app=(ECApp *)clientData;

  if (objc!=2)
    {
      Tcl_WrongNumArgs(interp,1,objv,"processID");
      return TCL_ERROR;
    }

  ECProcess* p=app->findECProcess(Tcl_GetString(objv[1]));
  if (p==NULL)
    {
      Tcl_AppendResult(interp,"invalid process ID",NULL);
      return TCL_ERROR;     
    }

  p->activate(app->getEvent());
  return TCL_OK;
}


// ---- Tcl terminate command
//

int ECTk_TerminateCmd(ClientData clientData, Tcl_Interp *interp, 
		      int objc,Tcl_Obj *const objv[])
{
  ECApp* app=(ECApp *)clientData;

  if (objc!=2)
    {
      Tcl_WrongNumArgs(interp,1,objv,"processID");
      return TCL_ERROR;
    }

  ECProcess* p=app->findECProcess(Tcl_GetString(objv[1]));
  if (p==NULL)
    {
      Tcl_AppendResult(interp,"invalid process ID",NULL);
      return TCL_ERROR;     
    }
  p->terminate();
  app->getEvent()->setCmdEvent(CmdProcessTerminated,NULL);
  return TCL_OK;
}


// ---- Tcl remove command
//

int ECTk_RemoveCmd(ClientData clientData, Tcl_Interp *interp, 
		   int objc,Tcl_Obj *const objv[])
{
  ECApp* app=(ECApp *)clientData;

  if (objc!=2)
    {
      Tcl_WrongNumArgs(interp,1,objv,"processID");
      return TCL_ERROR;
    }

  ECProcess* p=app->findECProcess(Tcl_GetString(objv[1]));
  if (p!=NULL)
    {
      if (app->getEvent()->cellStatus()==CellOn)
	{
	  Tcl_AppendResult(interp,
			   "can't remove a process while cell is ON",NULL);
	  return TCL_ERROR; 
	}
      if (strcmp(p->getProcessOwner(),"system")==0)
	{
	  Tcl_AppendResult(interp,"can't remove a system process",NULL);
	  return TCL_ERROR;     
	}
      
      app->removeECProcess(p);
      return TCL_OK;
    }

  // Check if one tries to remove an ECRec
  ECRec* rec=app->findECRecord(Tcl_GetString(objv[1]));
  if (rec==NULL)
    {
      Tcl_AppendResult(interp,"invalid process or record ID",NULL);
      return TCL_ERROR;     
    }
  
  app->removeECRecord(rec);
  return TCL_OK;
}


// ---- ECTK_EVENTLOOP : dispatches the ECTk events
//
// A Tcl_TimerProc used to schedule the ECTk event loop inside 
// the Tcl event loop
//
// The Timer register itself again continously
//

void ECTk_eventLoop(ClientData clientData)
{
  ECApp* app=(ECApp *)clientData;
  ECProcess* p=app->getFirstECProcess();
  while (p!=NULL)
    {
      p->handleEvent(app->event);
      p=p->getNextECProcess();
    }
  Tcl_CreateTimerHandler(app->output_update,ECTk_eventLoop,clientData);
}


// ----- ECAPP class functions ---------------------------------------
//
//


// ---- ECAPP : constructor
//

ECApp::ECApp(int argc, char **argv)
{
  Tcl_FindExecutable(argv[0]);

  interp=Tcl_CreateInterp();
  Tcl_Preserve((ClientData) interp);
  Tcl_SetVar(interp,"tcl_interactive","1",TCL_GLOBAL_ONLY);

  Tcl_Init(interp);
  Tk_SafeInit(interp);

  registerGlobalVars();
  registerNewCmd();
  registerUserCmd();

  event = new ECEvent(interp);
  FirstECProcess = new ECIO(interp);
  FirstECRecord = NULL;

  cout << "\n";
  cout << "\tE C T k\n";
  cout <<"\tThe electrochemical tool kit interface\n";
  cout << "\tVersion 1.0\n";
  cout << "\tlast modified July 2010\n";
  cout << "\n";
  cout << "\tCopyright (C) 2010\n";
  cout << "\tRolf Wuthrich, Concordia University, Canada\n";
  cout << "\tThis is free software; type 'puts $::ectk::license' for more details.\n";
  cout << "\n";
  cout << "\tFor help see the man pages.\n";
  cout << "\n";
  cout.flush();

  Tcl_CreateTimerHandler(output_update,ECTk_eventLoop,this);
  sourceRcFile();
}


// ---- ~ECAPP : destructor
//

ECApp::~ECApp()
{
  Tcl_Free(ectk_commentStr);

  ECProcess* Current=FirstECProcess;
  while(Current!=NULL)
    {
      Current=Current->getNextECProcess();
      delete FirstECProcess;
      FirstECProcess=Current;
    }

   ECRec* cur=FirstECRecord;
   while(cur!=NULL)
    {
      cur=cur->getNextECRec();
      delete FirstECRecord;
      FirstECRecord=cur;
    }

  delete event;

  Tcl_Release((ClientData) interp);
  Tcl_DeleteInterp(interp);

  cout << "\nExiting ElChemTk ...\n\n";
}


// ---- SourceRcFile : sources the config files
//

void ECApp::sourceRcFile()
{
  Tcl_EvalFile(interp,"//usr//local//share//ectk//elchemtkrc");
  Tcl_EvalFile(interp,"~//ectkrc.tcl");
}


// ---- REGISTERUSERCMD : registers new Tcl commands defined by user
//                        has to be defined by child

void ECApp::registerUserCmd()
{
}


// ---- REGISTERNEWCMD : registers new Tcl commands specific to ECApp
//

void ECApp::registerNewCmd()
{
  Tcl_CreateObjCommand(interp,"ps", ECTk_PsCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"lrec", ECTk_lrecCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"start", ECTk_StartCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"stop", ECTk_StopCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"activate", ECTk_ActivateCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"terminate", ECTk_TerminateCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"remove", ECTk_RemoveCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"ecrec", ECTk_ECRecCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"cell", ECTk_CellCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"trigger", ECTk_TriggerCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"cv", ECTk_CvCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"ca", ECTk_CACmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"step", ECTk_StepCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"pulse", ECTk_PulseCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateObjCommand(interp,"userproc", ECTk_UserProcCmd, 
		    (ClientData) this, (Tcl_CmdDeleteProc *) NULL);
}


// ---- REGISTERGLOBALVARS : registers global Tcl variables specific to ECApp
//

void ECApp::registerGlobalVars()
{
  Tcl_CreateNamespace(interp,"ectk", NULL, NULL);
  
  Tcl_LinkVar(interp,"::ectk::license",
              (char *)&ectk_license, TCL_LINK_STRING | TCL_LINK_READ_ONLY);
  
  ectk_commentStr=Tcl_Alloc(2);
  strcpy(ectk_commentStr,"#");
  Tcl_LinkVar(interp,"::ectk::commentStr",(char *)&ectk_commentStr,
	      TCL_LINK_STRING);

  output_update=1;
  Tcl_LinkVar(interp,"::ectk::IOboardUpdateInterval",(char *)&output_update,
  	      TCL_LINK_INT);
}


// ---- GETFIRSTECPROCESS : returns adress to the first ECProcess
//

ECProcess* ECApp::getFirstECProcess()
{
  return FirstECProcess;
}


// ---- ADDECPROCESS :  adds a new ECProcess to the process list
//

void ECApp::addECProcess(ECProcess* proc)
{
  ECProcess* p=FirstECProcess;

  while (p->getNextECProcess()!=NULL)
    p=p->getNextECProcess();

  p->setNextECProcess(proc);
  proc->setNextECProcess(NULL);
  Tcl_SetVar2(interp,"::ectk::procList",proc->getID(),proc->getTclCmd(),0);
}


// ---- REMOVEECPROCESS : removes a ECProcess from the list
//

void ECApp::removeECProcess(ECProcess* proc)
{
  ECProcess* p=FirstECProcess;
  while (p->getNextECProcess()!=NULL)
    {
      if (p->getNextECProcess()==proc) break;
      p=p->getNextECProcess();
    }
  p->setNextECProcess(proc->getNextECProcess());

  delete proc;
}


// ---- FINDECPROCESS :  finds the ECProcess with ID PID
//                       returns NULL if the process was not found

ECProcess* ECApp::findECProcess(const char* PID)
{
  ECProcess* p=FirstECProcess;
  while (p!=NULL)
    {
      if (strcmp(p->getID(),PID)==0) break;
      p=p->getNextECProcess();
    }
  return p;
}


// ---- FINDECPROCEDURE : finds the ECProcedure with ID PID
//                        returns NULL if the procedure was not found

ECProcedure* ECApp::findECProcedure(const char* PID)
{
  ECProcess* p=findECProcess(PID);
  if (p==NULL) return NULL;
  if (p->getNbrECRecords()>=0)
    return (ECProcedure *)p;
  else
    return NULL;
}


// ---- GETFIRSTECRECORD : returns adress to the first ECRecord
//

ECRec* ECApp::getFirstECRecord()
{
  return FirstECRecord;
}


// ---- ADDECRECORD : adds an ECRecord to the list
//

void ECApp::addECRecord(ECRec* rec, char* recCmd)
{
  ECRec* p=FirstECRecord;
  if (p==NULL)
    FirstECRecord=rec;
  else
    {
      while (p->getNextECRec()!=NULL)
	p=p->getNextECRec();
      p->setNextECRec(rec);
    }
  rec->setNextECRec(NULL);

  Tcl_SetVar2(interp,"::ectk::recList",rec->getID(),recCmd,0);
}


// ---- REMOVEECRECORD : removes a ECRecord from the list
//

void ECApp::removeECRecord(ECRec* rec)
{
  if (rec==FirstECRecord)
    {
      FirstECRecord=rec->getNextECRec();
      delete rec;
      return;
    }

  ECRec* p=FirstECRecord;
  while (p->getNextECRec()!=NULL)
    {
      if (p->getNextECRec()==rec) break;
      p=p->getNextECRec();
    }
  p->setNextECRec(rec->getNextECRec());
  delete rec;
}


// ---- FINDECRECORD : finds the ECRecord with ID recID
//                     returns NULL if the record was not found

ECRec* ECApp::findECRecord(const char* recID)
{
  ECRec* p=FirstECRecord;
  while (p!=NULL)
    {
      if (strcmp(p->getID(),recID)==0) break;
      p=p->getNextECRec();
    }
  return p;
}


// ---- RUN : main loop
//

void ECApp::run()
{
  Tcl_Channel inChannel  = Tcl_GetStdChannel(TCL_STDIN);
  Tcl_Channel outChannel = Tcl_GetStdChannel(TCL_STDOUT);

  Tcl_DString cmdline;
  Tcl_DStringInit(&cmdline);

  Tcl_DString prompt;
  Tcl_DStringInit(&prompt);
  Tcl_DStringAppend(&prompt,"ECTk > ",-1);

  Tcl_WriteChars(outChannel,Tcl_DStringValue(&prompt),-1);
  Tcl_Flush(outChannel);

  while (1)
    {
      inChannel = Tcl_GetStdChannel(TCL_STDIN);

      Tk_DoOneEvent(TK_DONT_WAIT);
      Tcl_DoOneEvent(TCL_DONT_WAIT);
     
      if (Tcl_Gets(inChannel, &cmdline)<0)
	{
	  if (Tcl_InputBlocked(inChannel)) 
	    {
	      continue;
	    }
	}

      if (Tcl_CommandComplete(Tcl_DStringValue(&cmdline)))
	{
	  Tcl_RecordAndEval(interp,Tcl_DStringValue(&cmdline),0);
	  if (strcmp(interp->result,"ECTk_Exit")==0)
	    {
	      break;
	    }
	  if (*interp->result!=0)
	    {
	      Tcl_WriteChars(outChannel,interp->result,-1);
	      Tcl_WriteChars(outChannel,"\n",1);
	    }

	  Tcl_DStringInit(&cmdline);
	  Tcl_WriteChars(outChannel,Tcl_DStringValue(&prompt),-1);
	  Tcl_Flush(outChannel);
	}
      else       // tcl command is not yet complete
	{
	  Tcl_WriteChars(outChannel,"\t",1);
	  Tcl_Flush(outChannel);
	}
    }

  Tcl_DStringFree(&prompt);
  Tcl_DStringFree(&cmdline);
}


// ---- GETEVENT : returns the adress of the event from the event loop
//

ECEvent* ECApp::getEvent()
{
  return event;
}
