// CVS ID : $Id: ECCA.cc,v 1.9 2010/12/05 20:14:48 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECCA
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.9 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#include "ECCA.h"

using namespace std;

// -------------------------------------------------------------------------
//
// Tcl interface


// Command to create a new CA object
//

int ECTk_CACmd(ClientData clientData, Tcl_Interp *interp, 
	       int objc,Tcl_Obj *const objv[])
{
  ECCA* ca  = new ECCA(interp);
  return ECTk_CreateECProcessCmd(interp,clientData,ca,objc,objv);
}
 

// -------------------------------------------------------------------------


// -----------------------------------------------------------------------
//
// class ECCA : chronoamperometry
//


// ---- HANDLEEVENT : define how ECCA reacts to external events
//

void ECCA::handleEvent(ECEvent* ev)
{
  ECProcedure::handleEvent(ev);

  // if not Active nothing to do
  if (status!=Active) return;

  // checks if experiment is finished
  if ((timeSinceActivated()>=stepTime+stepDuration)&&(status==Active))
    {
      terminate();
      ev->setCmdEvent(CmdProcessTerminated,this);
    }

  static int stepDone=0;
  char str[80];
  char dbl[TCL_DOUBLE_SPACE];

  // checks if the procedure was activated
  if (ev->cmd()==CmdProcedureActivated)
    {
      setMarkStr("t1");
      stepDone=0;
      if (verbose==1)
	{
	  Tcl_PrintDouble(interp,E1,dbl);
	  sprintf(str,"cons \"\t -- set potential to %s (V)\"",dbl); 
	  Tcl_Eval(interp,str);
	}
    }

  // checks if the step took place
  if ((timeSinceActivated()>=stepTime)&&(stepDone==0))
    {
      setMarkStr("t2");
      stepDone=1;
      if (verbose==1)
	{
	  Tcl_PrintDouble(interp,E2,dbl);
	  sprintf(str,"cons \"\t -- set potential to %s (V)\"",dbl); 
	  Tcl_Eval(interp,str);
	}
    }
}


// ---- POTENTIAL : Computes E(t)
//                  Implements a potential step from E1 to E2 at t=stepTime 

double ECCA::potential()
{
  if (timeSinceActivated()<=stepTime)
    return E1;
  else
    return E2;
}


// ---- INFO : returns a Tcl_DString containing relevant info about 
//             the electrochemical procedure

Tcl_DString* ECCA::info(const char* frontStr)
{
  ECProcedure::info(frontStr);
  addInfo(frontStr,"step time (s) = ",stepTime);
  addInfo(frontStr,"step duration (s) = ",stepDuration);
  addInfo(frontStr,"initial potential (V) = ",E1);
  addInfo(frontStr,"final potential (V) = ",E2);
  return infoStr;
}
