// This may look like C code, but it is really -*- C++ -*-
// CVS ID : $Id: ECCA.h,v 1.8 2010/09/07 00:48:05 wuthrich Exp $
// ---------------------------------------------------------------- 
//                                                                  
// Header file for  class ECCA
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.8 $)
//
// This software is copyright under the BSD license
//
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
//                                                                  
// ---------------------------------------------------------------

#ifndef ECCA_SEEN_
#define ECCA_SEEN_

#include <tcl.h>
#include "ECProcedure.h"

using namespace std;


// -----------------------------------------------------------------------
//
// Tcl interface
//

int ECTk_CACmd(ClientData clientData, Tcl_Interp *interp, 
	       int objc,Tcl_Obj *const objv[]);


// -----------------------------------------------------------------------
//
// class ECCA : chronoamperometry
//

class ECCA : public ECProcedure
{

protected:

  double stepTime;     // step time (duration of holding E1) in sec
  double stepDuration; // step duration (duration of holding E2) in sec
  double E1;           // start potential in Volts
  double E2;           // end potential in Volts

public:

  ECCA(Tcl_Interp* Interp) :
    ECObj("Chronoamperometry","ca",Interp),
    ECProcedure("Chronoamperometry","ca",Interp),
    stepTime(0),stepDuration(0),E1(0),E2(0)
  {
    addOption("-stepTime",&stepTime,"returns/sets time of step");
    addOption("-stepDuration",&stepDuration,"returns/sets step duration");
    addOption("-eInitial",&E1,"returns/sets potential before step");
    addOption("-eFinal",&E2,"returns/sets potential after step");
  }

  virtual void handleEvent(ECEvent* ev);
  virtual double potential();
  virtual Tcl_DString* info(const char* frontStr = "");
};

#endif /* ECCA_SEEN_ */
