// This may look like C code, but it is really -*- C++ -*-
// CVS ID : $Id: ECCV.h,v 1.11 2011/07/09 15:15:01 wuthrich Exp $
// ---------------------------------------------------------------- 
//                                                                  
// Header file for  class ECCV
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.11 $)
//
// This software is copyright under the BSD license
//
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#ifndef ECCV_SEEN_
#define ECCV_SEEN_

#include <tcl.h>
#include "ECProcedure.h"

using namespace std;


// -----------------------------------------------------------------------
//
// Tcl interface
//

int ECTk_CvCmd(ClientData clientData, Tcl_Interp *interp, 
	       int objc,Tcl_Obj *const objv[]);


// -----------------------------------------------------------------------
//
// class ECCV : cyclic voltammetry
//

class ECCV : public ECProcedure
{

protected:

  double scanRate;   // scan rate in V/s
  double E1;         // start potential in V
  double E2;         // end potential in V
  int    N;          // number of cycles to be performed

public:

  ECCV(Tcl_Interp* Interp) : 
    ECObj("Cylic Voltammetry","cv",Interp),
    ECProcedure("Cylic Voltammetry","cv",Interp),
    scanRate(0), E1(0), E2(0), N(0)
  {
    addOption("-scanRate",&scanRate,"returns/sets scan rate");
    addOption("-eInitial",&E1,"returns/sets initial potential");
    addOption("-eFinal",&E2,"returns/sets final potential");
    addOption("-nCycles",&N,"returns/sets number of cycles");

    Tcl_DStringFree(xCoord);
    Tcl_DStringAppend(xCoord,"$E",-1);
    Tcl_DStringFree(yCoord);
    Tcl_DStringAppend(yCoord,"$I",-1);
  }

  virtual void handleEvent(ECEvent* ev);
  virtual double potential();
  virtual Tcl_DString* info(const char* frontStr = "");
};


#endif /* ECCV_SEEN_ */
