// CVS ID : $Id: ECCell.cc,v 1.8 2010/07/21 18:19:58 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECCell
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.8 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#include <tcl.h>
#include "ECCell.h"
#include "ECApp.h"

using namespace std;


// -------------------------------------------------------------------------
//
// Tcl interface


// Command to create a new cell object
//

int ECTk_CellCmd(ClientData clientData, Tcl_Interp *interp, 
		 int objc,Tcl_Obj *const objv[])
{
  Tcl_ResetResult(interp);

  if (objc==1)
    {
      Tcl_WrongNumArgs(interp,1,objv,"-on\" or \"cell -off");
      return TCL_ERROR;
    }

  ECCell* cell = new ECCell(interp,CellOn);
  return ECTk_CreateECProcessCmd(interp,clientData,cell,objc,objv);
}


// -------------------------------------------------------------------------


// -----------------------------------------------------------------------
//
// class ECCell
//


// ---- CMD : process a potential command option (# i of objv[])
//            searches the options table 
//            if a match is found process the command option and
//            returns the index from the options table
//            if no match returns -1

int ECCell::cmd(int &i, int objc,Tcl_Obj *const objv[])
{
  // first executes the command options defined in ECProcess
  int index=ECProcess::cmd(i,objc,objv);

  if (index==getOptionIndex("-on")) 
    {
      cellStatus=CellOn;
      Tcl_DStringInit(preProcessScript);
      Tcl_DStringAppend(preProcessScript,"cons \"Turning cell ON\"",-1);
      name = "Cell ON ";
      i++;
    }

  if (index==getOptionIndex("-off"))
   {
      cellStatus=CellOff;
      Tcl_DStringInit(preProcessScript);
      Tcl_DStringAppend(preProcessScript,"cons \"\nTurning cell OFF\"",-1);
      name = "Cell OFF";
      i++;
    }

  if (((index==getOptionIndex("-preProcess"))&&(cellStatus==CellOn)))
    {
      Tcl_AppendResult(interp,
		       "preProcess not allowed for a cell -on object",
		       NULL);
      return -1;	  
    }

  if (((index==getOptionIndex("-postProcess"))&&(cellStatus==CellOff)))
    {
      Tcl_AppendResult(interp,
		       "postProcess not allowed for a cell -off object",
		       NULL);
      return -1;	  
    }

  return index;
}


// ---- HANDLEEVENT : define how ECCell reactes to external events
//

void ECCell::handleEvent(ECEvent* ev)
{
  ECProcess::handleEvent(ev);

  // case for a CELL ON object
  if ((status==Waiting)&&(ev->cmd()==CmdStart)&&(cellStatus==CellOn))
    {
      activate(ev);
      terminate();
      ev->setCellStatus(CellOn);
      ev->setCmdEvent(CmdCellOn,this);
    }

  // cases for a CELL OFF object
  if(cellStatus==CellOff)
    {
      if ((status==Waiting)&&(ev->cmd()==CmdProcessTerminated))
	{
	  activate(ev);
	  terminate();
	  ev->setCellStatus(CellOff);
	  ev->setCmdEvent(CmdCellOff,this);
	}

      if (ev->cmd()==CmdStop)
	{
	  ev->setCmdEvent(CmdCellOff,this);
	}
    }
}


// ---- INFO : returns a Tcl_DString containing relevant info about content
// 

Tcl_DString* ECCell::info(const char* frontStr)
{
  Tcl_DStringFree(infoStr);
  Tcl_DStringAppend(infoStr,frontStr,-1);
  if (cellStatus==CellOn) 
    Tcl_DStringAppend(infoStr,"Turn cell ON",-1);
  else 
    Tcl_DStringAppend(infoStr,"Turn cell OFF",-1);
  return infoStr;
}
