// CVS ID : $Id: ECEvent.cc,v 1.4 2010/08/10 00:46:37 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECEvent
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.4 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#include "ECEvent.h"

using namespace std;


// -----------------------------------------------------------------------
//
// class ECEvent : class for event manipulation and storing
//

// ---- destructor

ECEvent::~ECEvent()
{
  Tcl_UnlinkVar(interp,"t");
  Tcl_UnlinkVar(interp,"E");
  Tcl_UnlinkVar(interp,"I");
  Tcl_UnlinkVar(interp,"Q");
  Tcl_UnlinkVar(interp,"Qpos");
  Tcl_UnlinkVar(interp,"Qneg");
}


// ---- eventGenerator : returns the adress to the generator of the event
//                       returns NoCmd if eventType!=CmdEvent

ECProcess* ECEvent::eventGenerator()
{
  if (eventType==CmdEvent)
    return generator;
  else
    return NoCmd;
}


// ---- reset : resets the event to NoEvent
//              does not affect the electrochemical data

void ECEvent::reset()
{
  eventType=NoEvent;
  generator=NULL;
  EvCmd=NoCmd;
}


// ---- setCmdEvent : defines a command event

void ECEvent::setCmdEvent(int Cmd, ECProcess* EventGenerator)
{
  eventType=CmdEvent;
  EvCmd=Cmd;
  generator=EventGenerator;
}


// ---- resetQ : resets Q, Qpos and Qneg to 0

void ECEvent::resetQ()
{
  Q=0.0;
  Qpos=0.0;
  Qneg=0.0;
}
