// This may look like C code, but it is really -*- C++ -*-
// CVS ID : $Id: ECEvent.h,v 1.5 2010/08/10 00:46:37 wuthrich Exp $
// ---------------------------------------------------------------- 
//                                                                  
// Header file for  class ECEvent
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.5 $)
//
// This software is copyright under the BSD license
//
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#ifndef ECEvent_SEEN_
#define ECEvent_SEEN_

#include <tcl.h>

// -----------------------------------------------------------------------
//
// Event types
//

const int
  NoEvent  = 0,
  CmdEvent = 1;


// -----------------------------------------------------------------------
//
// Predefined commands
//

const int
  NoCmd                 = 0,
  CmdExit               = 1,
  CmdStart              = 2,
  CmdStop               = 3,
  CmdProcessTerminated  = 4,
  CmdProcedureActivated = 5,
  CmdCellOn             = 6,
  CmdCellOff            = 7;


// ------------------------------------------------------------------------
//
// Cell status
//

const int
  CellOn  = 1,
  CellOff = 0;


// -----------------------------------------------------------------------
//
// class ECEvent : class for event manipulation and storing
//

class ECProcess; // forward definition

class ECEvent
{

protected:

  Tcl_Interp *interp;

  int  eventType;
  ECProcess* generator;
  int  EvCmd;

  // data related to electrochemical experiments
  double t;        // time [s]
  double E;        // potential [V]
  double I;        // current [A]
  double Q;        // charge (since started or reseted) [C]
  double Qpos;     // positive charge (since started or reseted) [C]
  double Qneg;     // negative charge (since started or reseted) [C]

  // cell status
  int CellStatus;

public:

  ECEvent(Tcl_Interp* Interp) :
    interp(Interp),
    eventType(NoEvent), 
    generator(NULL),
    t(0), E(0), I(0), Q(0), Qpos(0), Qneg(0), 
    CellStatus(CellOff)
  {
    Tcl_LinkVar(Interp,"t",(char *)&t,TCL_LINK_DOUBLE|TCL_LINK_READ_ONLY);
    Tcl_LinkVar(Interp,"E",(char *)&E,TCL_LINK_DOUBLE);
    Tcl_LinkVar(Interp,"I",(char *)&I,TCL_LINK_DOUBLE|TCL_LINK_READ_ONLY);
    Tcl_LinkVar(Interp,"Q",(char *)&Q,TCL_LINK_DOUBLE|TCL_LINK_READ_ONLY);
    Tcl_LinkVar(Interp,"Qpos",(char *)&Qpos,TCL_LINK_DOUBLE|TCL_LINK_READ_ONLY);
    Tcl_LinkVar(Interp,"Qneg",(char *)&Qneg,TCL_LINK_DOUBLE|TCL_LINK_READ_ONLY);
  }
  ~ECEvent();

  int    cmd() {return EvCmd;}
  ECProcess* eventGenerator();
  void   reset();
  void   setCmdEvent(int Cmd, ECProcess* EventGenerator);

  void   setEventType(int type) {eventType=type;}
  void   setCellStatus(int status) {CellStatus=status;}
  int    cellStatus() {return CellStatus;}

  void   resetQ();
  void   setT(double T) {t=T;}
  void   setE(double potential) {E=potential;}
  void   setI(double current) {I=current;}
  void   setQ(double charge) {Q=charge;}
  void   setQpos(double PosCharge) {Qpos=PosCharge;}
  void   setQneg(double NegCharge) {Qneg=NegCharge;}
  double getT() {return t;}
  double getE() {return E;}
  double getI() {return I;}
  double getQ() {return Q;}
  double getQpos() {return Qpos;}
  double getQneg() {return Qneg;}
};

#endif /* ECEvent_SEEN_ */
