// CVS ID : $Id: ECIO.cc,v 1.13 2011/07/09 15:15:01 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECIO
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.13 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#include <tcl.h>
#include <comedilib.h>
#include <comedi.h>
#include <math.h>
#include <cstring>
#include "ECEvent.h"
#include "ECIO.h"

using namespace std;


// ------------------------------------------------------------
//
// New Tcl commands defined by ECIO
//


// ---- Tcl IODevice command
//

int ECTk_IOdeviceCmd(ClientData clientData, Tcl_Interp *interp, 
		     int objc,Tcl_Obj *const objv[])
{
  ECIO* io=(ECIO *)clientData;
  int i;
  double x;
  char str[TCL_DOUBLE_SPACE];
  Tcl_ResetResult(interp);

  if (objc==1)
    {
      Tcl_WrongNumArgs(interp,1,objv,"subcommand ?argument ...?");
      return TCL_ERROR;
    }

  int index;
  static CONST char* cmds[] = {"-connect","-board","-driver","-deviceFile",
			       "-read","-write",NULL};
  if (Tcl_GetIndexFromObj(interp,objv[1],cmds,"subcommand",'0',&index)!=TCL_OK)
    return TCL_ERROR;

  switch (index)
    {
    case 0: // -connect
      if (objc>3)
	{
	  Tcl_WrongNumArgs(interp,2,objv,"?configFile?");
	  return TCL_ERROR;
	}
      if (objc==3) Tcl_EvalFile(interp,Tcl_GetString(objv[2]));
      if (io->connectToIOCard()==-1) return TCL_ERROR;
      break;

    case 1: // -driver
      if (objc!=2)
	{
	  Tcl_WrongNumArgs(interp,2,objv,NULL);
	  return TCL_ERROR;
	}
      Tcl_AppendResult(interp,comedi_get_driver_name(io->iocard->device),NULL);
      break;

    case 2: // -board
      if (objc!=2)
	{
	  Tcl_WrongNumArgs(interp,2,objv,NULL);
	  return TCL_ERROR;
	}
      Tcl_AppendResult(interp,comedi_get_board_name(io->iocard->device),NULL);
      break;

    case 3: // -deviceFile
      if (objc>3)
	{
	  Tcl_WrongNumArgs(interp,2,objv,"?comediFile?");
	  return TCL_ERROR;
	}
      if (objc==3) 
	Tcl_SetVar(interp,"::ectk::comediFile",Tcl_GetString(objv[2]),0);
      Tcl_AppendResult(interp,Tcl_GetVar(interp,"::ectk::comediFile",0),NULL);
      break;

    case 4: // -read
      if (objc!=3)
	{
	  Tcl_WrongNumArgs(interp,2,objv,"chan#");
	  return TCL_ERROR;
	}
      if (Tcl_GetIntFromObj(interp,objv[2],&i)!=TCL_OK) return TCL_ERROR;
      io->iocard->readData(i,x)!=0;
      Tcl_PrintDouble(interp,x,str);
      Tcl_AppendResult(interp,str,NULL);
      break;

    case 5: // -write
      if (objc!=4)
	{
	  Tcl_WrongNumArgs(interp,2,objv,"chan# data");
	  return TCL_ERROR;
	}	
      if (Tcl_GetIntFromObj(interp,objv[2],&i)!=TCL_OK) return TCL_ERROR;
      if (Tcl_GetDoubleFromObj(interp,objv[3],&x)!=TCL_OK) return TCL_ERROR;
      io->iocard->writeData(i,x);
      break;
    }

  return TCL_OK;
}


// ---- Tcl graph command
//

int ECTk_GraphCmd(ClientData clientData, Tcl_Interp *interp, 
		  int objc,Tcl_Obj *const objv[])
{
  ECIO* io=(ECIO *)clientData;

  int index=0;
  static CONST char *opts[] = {"-show","-hide","-stop","-x","-y","-style",NULL};

  if (objc==1) io->graphStatus=GraphShow;

  int i=1;
  while (i<objc)
    {
      if (Tcl_GetIndexFromObj(interp,objv[i],opts,"option",'0',&index)!=TCL_OK)
	return TCL_ERROR;

      switch (index)
	{
	case 0: // -show
	  io->graphStatus=GraphShow;
	  i++;
	  break;

	case 1: // -hide
	  io->graphStatus=GraphHide;
	  i++;
	  break;
	  
	case 2: // -stop
	  io->graphStatus=GraphOff;
	  i++;
	  break;

	case 3: // -x
	  if ((i+1<objc)&&(Tcl_StringMatch(Tcl_GetString(objv[i+1]),"-*")==0))
	    {
	      Tcl_DStringFree(io->xCoord);
	      Tcl_DStringAppend(io->xCoord,Tcl_GetString(objv[i+1]),-1);
	      i=i+2;
	    }
	  else
	    {
	      Tcl_AppendResult(interp,Tcl_DStringValue(io->xCoord),NULL);
	      i++;
	    }
	  break;

	case 4: // -y
	  if ((i+1<objc)&&(Tcl_StringMatch(Tcl_GetString(objv[i+1]),"-*")==0))
	    {
	      Tcl_DStringFree(io->yCoord);
	      Tcl_DStringAppend(io->yCoord,Tcl_GetString(objv[i+1]),-1);
	      i=i+2;
	    }
	  else
	    {
	      Tcl_AppendResult(interp,Tcl_DStringValue(io->yCoord),NULL);
	      i++;
	    }
	  break;

	case 5: // -style
	  if ((i+1<objc)&&(Tcl_StringMatch(Tcl_GetString(objv[i+1]),"-*")==0))
	    {
	      Tcl_DStringFree(io->plotStyle);
	      Tcl_DStringAppend(io->plotStyle,Tcl_GetString(objv[i+1]),-1);
	      i=i+2;
	    }
	  else
	    {
	      Tcl_AppendResult(interp,Tcl_DStringValue(io->plotStyle),NULL);
	      i++;
	    }
	  break;

	}
    }

  return TCL_OK;
}


// ---- Tcl IOStat command
//

int ECTk_IOStatCmd(ClientData clientData, Tcl_Interp *interp, 
		   int objc,Tcl_Obj *const objv[])
{
  ECIO* io=(ECIO *)clientData;

  if (objc!=1)
    {
      Tcl_WrongNumArgs(interp,1,objv,NULL);
      return TCL_ERROR;
    }

  Tcl_ResetResult(interp);
  char   str[80];

  sprintf(str,"# IO operations : %d\n",io->n);
  Tcl_AppendResult(interp,str,NULL);

  Tcl_AppendResult(interp,"<dt> = ",NULL);
  Tcl_PrintDouble(interp,1000.0*(io->sum_dt/io->n),str);
  Tcl_AppendResult(interp,str," msec\n",NULL);

  Tcl_AppendResult(interp,"std dev = ",NULL);
  Tcl_PrintDouble(interp,
      1000.0*sqrt(io->sqr_sum_dt-io->sum_dt*io->sum_dt/io->n)/(io->n-1),str);
  Tcl_AppendResult(interp,str," msec\n",NULL);

  Tcl_AppendResult(interp,"min = ",NULL);
  Tcl_PrintDouble(interp,1000.0*io->min,str);
  Tcl_AppendResult(interp,str," msec\n",NULL);

  Tcl_AppendResult(interp,"max = ",NULL);
  Tcl_PrintDouble(interp,1000.0*io->max,str);
  Tcl_AppendResult(interp,str," msec\n",NULL);
  return TCL_OK;
}



// ---------------------------------------------------------------
//
// class comediInterface
//


// ---- CONSTRUCTOR
//

comediInterface::comediInterface(const char* filename, 
				 chanGain* ao, chanGain* ai,
				 int chan_inpI, int chan_inpE, int chan_out)
  : emptyComediInterface(),
    chanIfo(chan_inpI),
    chanVfo(chan_inpE),
    chanCtrl(chan_out)
{
  range = 0;
  aref = AREF_GROUND;

  AI=ai;
  AO=ao;  

  device=comedi_open(filename);
  if (!device)
    {
      inp_subdev = comedi_find_subdevice_by_type(device,COMEDI_SUBD_AI,0);
      out_subdev = comedi_find_subdevice_by_type(device,COMEDI_SUBD_AO,0);

      maxdata = comedi_get_maxdata(device,inp_subdev,chan_inpI);
      cr      = comedi_get_range(device,inp_subdev,chan_inpI,range);

      writeData(chan_out,0.0+AO[chan_out].offset);

      // to avoid that data=min or data=max gives NAN
      comedi_set_global_oor_behavior(COMEDI_OOR_NUMBER);

      instr_list.n_insns=3;
      instr_list.insns=instr;

      instr[0].insn=INSN_READ;
      instr[0].n=1;
      instr[0].data=&inputE;
      instr[0].subdev=inp_subdev;
      instr[0].chanspec=CR_PACK(chan_inpE,range,aref);

      instr[1].insn=INSN_READ;
      instr[1].n=1;
      instr[1].data=&inputI;
      instr[1].subdev=inp_subdev;
      instr[1].chanspec=CR_PACK(chan_inpI,range,aref);

      instr[2].insn=INSN_WRITE;
      instr[2].n=1;
      instr[2].data=&output;
      instr[2].subdev=out_subdev;
      instr[2].chanspec=CR_PACK(chan_out,range,aref);
    }
}


// ---- GETDEVICE : returns device
//

comedi_t* emptyComediInterface::getDevice()
{
  return device;
}


// ---- IOOPERATION : proceeds with IO operations
//

int emptyComediInterface::ioOperation(double &E, double &I)
{
  I=0.0;
  return 3; // in order to simulate succesfull readout
}


// ---- IOOPERATION : proceeds with IO operations
//      
// returns number of succesfull operations done
//

int comediInterface::ioOperation(double &E, double &I)
{
  output=AO[chanCtrl].gain*comedi_from_phys(E,cr,maxdata)+AO[chanCtrl].offset;
  int ret=comedi_do_insnlist(device,&instr_list);
  I=AI[chanIfo].gain*comedi_to_phys(inputI,cr,maxdata)+AI[chanIfo].offset;
  E=AI[chanVfo].gain*comedi_to_phys(inputE,cr,maxdata)+AI[chanVfo].offset;
  return ret;
}


// ---- readData : reads data from a channel
//      

int emptyComediInterface::readData(int channel, double &data)
{
  data=0.0;
  return 0;
}


// ---- readData : reads data from a channel
//      returns -1 in case of an error
// 
//

int comediInterface::readData(int channel, double &data)
{
  lsampl_t d;
  int ret=comedi_data_read(device,inp_subdev,channel,range,aref,&d);
  data=AI[channel].gain*comedi_to_phys(d,cr,maxdata)+AI[channel].offset;
  return ret;
}


// ---- writeData : writes data to a channel
//      

int emptyComediInterface::writeData(int channel, double data)
{
  return 0;
}


// ---- writeData : writes data to a channel
//      
// returns -1 in case of an error
//

int comediInterface::writeData(int channel, double data)
{
  lsampl_t d;
  d=AO[channel].gain*comedi_from_phys(data,cr,maxdata)+AO[channel].offset;
  int ret=comedi_data_write(device, out_subdev, channel, range, aref, d);
  return ret;
}



// ---------------------------------------------------------------
//
// class ECIO : class responsible for AD/DA IO operations
//

// ---- CONSTRUCTOR
//

ECIO::ECIO(Tcl_Interp *Interp) :
  ECObj("IO Module","",Interp),
  ECProcess("IO Module","system","",Interp),
  activeECProcedure(NULL),
  graphStatus(GraphOff),
  monitorTime(0.0),
  saveTime(0.0),
  graphTime(0.0),
  dt_graphUpdate(0.5),
  dt_monitorUpdate(1.0)
{
  status=Active;

  // Tcl variables for monitor and graph updadte intervals
  Tcl_LinkVar(Interp,"::ectk::monitorUpdateInterval",(char*)&dt_monitorUpdate,
	      TCL_LINK_DOUBLE);
  Tcl_LinkVar(Interp,"::ectk::graphUpdateInterval",(char*)&dt_graphUpdate,
	      TCL_LINK_DOUBLE);

  // Tcl graph command
  dataFile.open(".ecgraph_tmp",ios::trunc);
  Tcl_CreateObjCommand(Interp,"graph",ECTk_GraphCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);
  xCoord     = new Tcl_DString;
  yCoord     = new Tcl_DString;
  plotStyle  = new Tcl_DString;
  plotString = new Tcl_DString;
  Tcl_DStringInit(xCoord);
  Tcl_DStringAppend(xCoord,"$t",-1);
  Tcl_DStringInit(yCoord);
  Tcl_DStringAppend(yCoord,"$E",-1);
  Tcl_DStringInit(plotStyle);
  Tcl_DStringAppend(plotStyle,"with lines notitle",-1);
  Tcl_DStringInit(plotString);
  
  // IO operation related configuration
  comediFile=Tcl_Alloc(13);
  strcpy(comediFile,"/dev/comedi0");
  Tcl_LinkVar(Interp,"::ectk::comediFile",(char *)&comediFile,TCL_LINK_STRING);
  iocard = new emptyComediInterface();
  Tcl_CreateObjCommand(Interp,"IODevice" ,ECTk_IOdeviceCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);
  chan_inpE = 1;
  chan_inpI = 2;
  chan_out  = 1;
  Tcl_LinkVar(Interp,"::ectk::chanIfo",(char*)&chan_inpI,TCL_LINK_INT);
  Tcl_LinkVar(Interp,"::ectk::chanVfo",(char*)&chan_inpE,TCL_LINK_INT);
  Tcl_LinkVar(Interp,"::ectk::chanCtrl",(char*)&chan_out,TCL_LINK_INT);
  int i;
  char str[25];
  for (i=0;i<16;i++)
    {
      AI[i].gain=1.0;
      AI[i].offset=0.0;
      sprintf(str,"::ectk::AI_gain(%i)",i);
      Tcl_LinkVar(Interp,str,(char *)&AI[i].gain,TCL_LINK_DOUBLE);
      sprintf(str,"::ectk::AI_offset(%i)",i);
      Tcl_LinkVar(Interp,str,(char *)&AI[i].offset,TCL_LINK_DOUBLE);
      AO[i].gain=1.0;
      AO[i].offset=0.0;
      sprintf(str,"::ectk::AO_gain(%i)",i);
      Tcl_LinkVar(Interp,str,(char *)&AO[i].gain,TCL_LINK_DOUBLE);
      sprintf(str,"::ectk::AO_offset(%i)",i);
      Tcl_LinkVar(Interp,str,(char *)&AO[i].offset,TCL_LINK_DOUBLE);
    }


  // initalisation of IO operation statistics
  min=1e6;
  max=0;
  sum_dt=0;
  sqr_sum_dt=0;
  n=0;
  Tcl_CreateObjCommand(Interp,"IOStat",ECTk_IOStatCmd, 
		       (ClientData) this, (Tcl_CmdDeleteProc *) NULL);
}


// ---- DESTRUCTOR
//

ECIO::~ECIO()
{
  dataFile.close();
  delete iocard;
  Tcl_DStringFree(xCoord);
  Tcl_DStringFree(yCoord);
  Tcl_DStringFree(plotStyle);
  Tcl_DStringFree(plotString);
  delete xCoord;
  delete yCoord;
  delete plotStyle;
  delete plotString;
}


// ---- HANDLEEVENT : define how ECIO reactes to external events
//

void ECIO::handleEvent(ECEvent* ev)
{
  // checks if Cell was turned ON
  if (ev->cmd()==CmdCellOn)
    {
      // starts a new statistics about IO operations
      min=1e6;
      max=0;
      sum_dt=0;
      sqr_sum_dt=0;
      n=0;

      // checks if ECTK is connected to a IO card
      if (iocard->getDevice()==NULL)
	Tcl_Eval(interp,"cons \"WARNING no IO card connected.\"");

      // needed in order the next process gets activated
      ev->setCmdEvent(CmdProcessTerminated,this);
    }

  // checks if a ECProcedure has been activated
  if (ev->cmd()==CmdProcedureActivated)
    {
      activeECProcedure=(ECProcedure *) ev->eventGenerator();
      ev->reset();
    }

  // checks if Cell was turned OFF
  if (ev->cmd()==CmdCellOff)
    {
      activeECProcedure=NULL;
      dataFile.close();
      dataFile.open(".ecgraph_tmp",ios::trunc);
      ev->reset();
    }

  // if there is no active ECProcedure: nothing more to do
  if (activeECProcedure==NULL) return;

  double t_old=ev->getT();
  double cur;
  double pot=activeECProcedure->potential();
  double t=activeECProcedure->timeSinceActivated();
  ev->setE(pot);  

  // sends the data to the IO device and reads the current and potential
  if (iocard->ioOperation(pot,cur)!=3)
    {
      IOerror();
      return;
    }
  
  double dt=t-t_old;
  saveTime=saveTime+dt;
  graphTime=graphTime+dt;
  monitorTime=monitorTime+dt;

  ev->setE(pot);
  ev->setI(cur);
  ev->setT(t);
  ev->setQ(ev->getQ()+cur*dt);
  if (cur>0)
    ev->setQpos(ev->getQpos()+cur*dt);
  else
    ev->setQneg(ev->getQneg()+cur*dt);

  addStatReco(dt);

  if (monitorTime>=dt_monitorUpdate)
    {
      monitorTime=0.0;
      Tcl_UpdateLinkedVar(interp,"E");
      Tcl_UpdateLinkedVar(interp,"I");
    }
      
  if (graphStatus!=GraphOff)
    {
      if (saveTime>=activeECProcedure->dtOut())
	{
	  saveTime=0.0;
	  Tcl_DStringFree(plotString);
	  Tcl_DStringAppend(plotString,"expr {",-1);
	  Tcl_DStringAppend(plotString,Tcl_DStringValue(xCoord),-1);
	  Tcl_DStringAppend(plotString,"}",-1);
	  Tcl_Eval(interp,Tcl_DStringValue(plotString));
	  dataFile << Tcl_GetStringResult(interp) << "\t";
	  Tcl_DStringFree(plotString);
	  Tcl_DStringAppend(plotString,"expr {",-1);
	  Tcl_DStringAppend(plotString,Tcl_DStringValue(yCoord),-1);
	  Tcl_DStringAppend(plotString,"}",-1);
	  Tcl_Eval(interp,Tcl_DStringValue(plotString));
	  dataFile << Tcl_GetStringResult(interp) << "\n";
	  Tcl_ResetResult(interp);
	}

      if ((graphTime>=dt_graphUpdate)&&graphStatus==GraphShow)
	{
	  graphTime=0.0;
	  dataFile.flush();
	  Tcl_DStringFree(plotString);
	  Tcl_DStringAppend(plotString,"plotGnuplot .ecgraph_tmp ",-1);
	  Tcl_DStringAppend(plotString,Tcl_DStringValue(plotStyle),-1);
	  Tcl_Eval(interp,Tcl_DStringValue(plotString));
	  Tcl_ResetResult(interp);
	}
    }
}


// ---- CONNECTOTIOCARD : connects the IO card via comedi
//

int ECIO::connectToIOCard()
{
  delete iocard;
  iocard = new comediInterface(comediFile,AI,AO,chan_inpI,chan_inpE,chan_out);
  if(!iocard->getDevice())
    {
      IOerror();
      return -1;
    }
  return 0;
}


// ---- IOERROR : called whenever an IO error occurs
//

void ECIO::IOerror()
{
  Tcl_DString errorStr;
  Tcl_DStringInit(&errorStr);
  Tcl_DStringAppend(&errorStr,"error \"IO error : ",-1);
  //Tcl_DStringAppend(&errorStr,comedi_strerror(comedi_errno()),-1);
  Tcl_DStringAppend(&errorStr,"\"",-1);
  Tcl_Eval(interp,Tcl_DStringValue(&errorStr));
  Tcl_DStringFree(&errorStr);
  delete iocard;
  iocard = new emptyComediInterface();
}


// ---- ADDSTATRECO : adds an entry to the statistics about IO operations
//

void ECIO::addStatReco(double dt)
{
  if (min>dt) min=dt;
  if (max<dt) max=dt;
  n++;
  sum_dt=sum_dt+dt;
  sqr_sum_dt=sqr_sum_dt+dt*dt;
}
