// This may look like C code, but it is really -*- C++ -*-
// CVS ID : $Id: ECIO.h,v 1.7 2011/03/08 01:33:28 wuthrich Exp $
// ---------------------------------------------------------------- 
//                                                                  
// Header file for  class ECIO
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.7 $)
//
// This software is copyright under the BSD license
//
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
//                                                                  
// ---------------------------------------------------------------

#ifndef ECIO_SEEN_
#define ECIO_SEEN_

#include <tcl.h>
#include <vector>
#include <comedilib.h>
#include <comedi.h>
#include "ECProcess.h"
#include "ECProcedure.h"

using namespace std;


// -----------------------------------------------------------------------
//
// Graph status
//

const int
  GraphOff  = 0,          // do not graph
  GraphHide = 1,          // do save values, but not display them
  GraphShow = 2;          // save and display data


// -------------------------------------------------------------------------
//
// Structure storing the gain and offset for IO channels

struct chanGain
{
  double gain;
  double offset;
};


// -----------------------------------------------------------------------
//
// class comediInterface
//

class emptyComediInterface
{
  friend int ECTk_IOdeviceCmd(ClientData clientData, Tcl_Interp *interp, 
			      int objc,Tcl_Obj *const objv[]);
protected:

  comedi_t*   device;         // I-O device

public:

  emptyComediInterface(): device(NULL){};

  comedi_t* getDevice();
  virtual int ioOperation(double &E, double &I);
  virtual int readData(int channel, double &data);
  virtual int writeData(int channel, double data);
};


class comediInterface : public emptyComediInterface
{
protected:

  int inp_subdev;
  int out_subdev;
  int range;
  int aref;

  chanGain* AO;         // AO channel gains and offsets
  chanGain* AI;         // AI channel gains and offsets
  int chanIfo;          // channel for current follower
  int chanVfo;          // channel for potential follower
  int chanCtrl;         // channel for control

  lsampl_t inputI;
  lsampl_t inputE;
  lsampl_t output;

  lsampl_t maxdata;
  comedi_range* cr;

  comedi_insn     instr[3];
  comedi_insnlist instr_list;

public:

  comediInterface(const char* filename,
		  chanGain* ao, chanGain* ai,
		  int chan_inpI, int chan_inpE, int chan_out);
  ~comediInterface()
  {
    comedi_close(device);
  }

  virtual int ioOperation(double &E, double &I);
  virtual int readData(int channel, double &data);
  virtual int writeData(int channel, double data);
};



// ---------------------------------------------------------------
//
// class ECIO : class responsible for AD/DA IO operations
//

class ECIO : public ECProcess
{
  friend int ECTk_GraphCmd(ClientData clientData, Tcl_Interp *interp, 
			   int objc,Tcl_Obj *const objv[]);
  friend int ECTk_IOdeviceCmd(ClientData clientData, Tcl_Interp *interp, 
			      int objc,Tcl_Obj *const objv[]);
  friend int ECTk_IOStatCmd(ClientData clientData, Tcl_Interp *interp, 
			    int objc,Tcl_Obj *const objv[]);

private:

  Tcl_DString* plotString;  // used to generate graphs

protected:

  ECProcedure* activeECProcedure;
  
  ofstream     dataFile;
  double       monitorTime;
  double       saveTime;
  double       graphTime;
  double       dt_monitorUpdate;
  double       dt_graphUpdate;
  int          graphStatus;
  Tcl_DString* xCoord;          // used for x coordinate
  Tcl_DString* yCoord;          // used for y coordinate
  Tcl_DString* plotStyle;       // used for configurating graph


  // IO interface
  emptyComediInterface* iocard;
  char*    comediFile;    // dev file for comedi
  int      chan_inpI;     // AI channel # for Ifo
  int      chan_inpE;     // AI channel # for Vfo
  int      chan_out;      // AO channel # for driver signal
  chanGain AI[16];        // AI gains and offsets charactersitics
  chanGain AO[16];        // AO gains and offsets charactersitics

  // entries for IO operation stats
  double   min;
  double   max;
  double   sum_dt;
  double   sqr_sum_dt;
  int      n;

public:

  ECIO(Tcl_Interp *Interp);
  ~ECIO();

  virtual void handleEvent(ECEvent* ev);

  int  connectToIOCard();
  void IOerror();
  void addStatReco(double dt);
};


#endif /* ECIO_SEEN_ */
