// This may look like C code, but it is really -*- C++ -*-
// CVS ID : $Id: ECObj.h,v 1.3 2010/09/07 00:48:05 wuthrich Exp $
// ---------------------------------------------------------------- 
//                                                                  
// Header file for class ECObj
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.3 $)
//
// This software is copyright under the BSD license
//
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 09.07.2010 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#ifndef ECObj_SEEN_
#define ECObj_SEEN_

#include <tcl.h>
#include <vector>

using namespace std;


// -------------------------------------------------------------------------
//
// Tcl interface

int ECTk_ECObjCmd(ClientData clientData, Tcl_Interp *interp, 
		  int objc,Tcl_Obj *const objv[]);


// -------------------------------------------------------------------------
//
// Structure needed to maintain list of options with associated variables

struct AssocVar
{
  ClientData     varPtr;
  int            varType;
  Tcl_DString*   help;
};

const int
  EC_NO_VAR = 0,
  EC_INT = 1,
  EC_DOUBLE = 2,
  EC_DSTRING = 3;


// -----------------------------------------------------------------------
//
// class ECObj : abstract class from which all EC objects derive
//

class ECObj
{
private:

  static int       IDcounter;
  vector<AssocVar> assVars;
  int              addOpt(const char* opt);

protected:

  char             objID[8];
  const char*      name;

  Tcl_DString*     TclCmd;
  char**           cmdOpts;
  Tcl_Interp*      interp;

  Tcl_DString*     infoStr;    // used for info method

public:

  ECObj(const char* objName, const char* objCmd, Tcl_Interp* Interp);
  virtual ~ECObj();

  int           addOption(const char* opt, const char* help);
  int           addOption(const char* opt, double* dblPtr, const char* help);
  int           addOption(const char* opt, int* intPtr, const char* help);
  int           addOption(const char* opt, Tcl_DString* DStrPtr, const char* help);
  int           getOptionIndex(const char* opt);
  const char**  optsTbl();
  virtual int   cmd(int &i,int objc,Tcl_Obj *const objv[]);
  const char*   getTclCmd();

  virtual Tcl_DString* info(const char* frontStr = "");
  void          addInfo(const char* frontStr,const char* text,const char* var);
  void          addInfo(const char* frontStr,const char* text,int var);
  void          addInfo(const char* frontStr,const char* text,double var);

  const char*   getID();
  const char*   getECObjName();

  virtual int   getNbrECRecords();
};

#endif /* ECObj_SEEN_ */
