// CVS ID : $Id: ECProcedure.cc,v 1.10 2010/09/07 00:48:05 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECProcedure
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.10 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// 20.06.2010 Added ECRec                      R. Wuthrich
// 27.06.2010 Moved ECRec                      R. Wuthrich
// ---------------------------------------------------------------

#include <tcl.h>
#include "ECProcedure.h"

using namespace std;


// -----------------------------------------------------------------------
//
// class ECProcedure : abstract class from which all EC procedures derive
//


// ---- CONSTRUCTOR
//

ECProcedure::ECProcedure(const char *procName, const char* procCmd, 
			 Tcl_Interp *Interp) :
  ECObj(procName,procCmd,Interp),
  ECProcess(procName,"user",procCmd,Interp),
  ECData(procName,Interp),
  dt_aquire(0.1)
{
  addOption("-dtRecord",&dt_aquire,
	    "returns/sets time interval between two records");
}


// ---- CMD : process a potential command option (# i of objv[])
//            searches the options table 
//            if a match is found processes the command option and
//            returns the index from the options table
//            if no match returns -1

int ECProcedure::cmd(int &i,int objc,Tcl_Obj *const objv[])
{
  // first executes the command options defined in ECProcess and ECData
  int index=ECProcess::cmd(i,objc,objv);
  if ((i<objc)&&(index!=-1)) index=ECData::cmd(i,objc,objv);

  return index;
}

 
// ---- HANDLEEVENT : define how ECProcedure reacts to external events
//                    Has to be completed by child

void ECProcedure::handleEvent(ECEvent* ev)
{
  ECProcess::handleEvent(ev);

  static double t_record=0;

  // if the procedure is active, records the data
  if ((status==Active)&&(ev->getT()>=t_record))
    {
      t_record=t_record+dt_aquire;
      addECRecord(ev->getT(),ev->getE(),ev->getI(),
                  ev->getQ(),ev->getQpos(),ev->getQneg());
    }

  // checks if the procedure should be activated
  if ((status==Waiting)&&(ev->cmd()==CmdProcessTerminated))
    {
      removeRecords();
      t_record=0;
      activate(ev);
      ev->setCmdEvent(CmdProcedureActivated,this);
    }
}


// ---- POTENTIAL : computes the potential E to be outputted at time t
//                  use t=timeSinceActivated() to compute t
//                  Has to be defined by child

double ECProcedure::potential()
{
  return 0.0;
}


// ---- INFO : returns a Tcl_DString containing relevant info about content
//             Has to be defined by child

Tcl_DString* ECProcedure::info(const char* frontStr)
{
  ECProcess::info(frontStr);
}


// ---- GETNBRECRECORDS : returns n
//                        can be used to distinguish from an ECProcess

int ECProcedure::getNbrECRecords()
{
  return n;
}
