// This may look like C code, but it is really -*- C++ -*-
// CVS ID : $Id: ECProcess.h,v 1.7 2010/08/10 00:46:37 wuthrich Exp $
// ---------------------------------------------------------------- 
//                                                                  
// Header file for  class ECProcess
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.7 $)
//
// This software is copyright under the BSD license
//
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// 20.06.2010 Added getNbrECRecords()          R. Wuthrich
// ---------------------------------------------------------------

#ifndef ECProcess_SEEN_
#define ECProcess_SEEN_

#include <sys/time.h>
#include <tcl.h>
#include "ECEvent.h"
#include "ECObj.h"

using namespace std;

// -----------------------------------------------------------------------
//
// Process status types
//

const int
  Waiting    = 0,         // process waiting to be executed
  Active     = 1,         // process under execution
  Terminated = 2;         // process terminated

const char StatusStr[3][11] = {"Waiting","Active","Terminated"};


// -------------------------------------------------------------------------
//
// Tcl interface

int ECTk_CreateECProcessCmd(Tcl_Interp *interp, ClientData clientData,
			    ECProcess* proc, int objc,Tcl_Obj *const objv[]);


// -----------------------------------------------------------------------
//
// class ECProcess : abstract class from which all EC processes derive
//

class ECProcess : public virtual ECObj
{
private:

  timeval      activationTime;

protected:

  const char*  owner;
  int          status; 
  int          verbose;

  Tcl_DString* preProcessScript;
  Tcl_DString* postProcessScript;

  ECProcess*   nextECProc;

public:

  ECProcess(const char* procName, const char* procOwner,
	    const char* procCmd,  Tcl_Interp* Interp);
  ~ECProcess();

  virtual int  cmd(int &i,int objc,Tcl_Obj *const objv[]);
  virtual void handleEvent(ECEvent* ev);
  virtual Tcl_DString* info(const char* frontStr = "");
  virtual int  getNbrECRecords();

  void         terminate();
  void         activate(ECEvent* ev);
  double       timeSinceActivated();

  void         setPreProcessScript(Tcl_DString* PreProcessScript);
  void         setPostProcessScript(Tcl_DString* PostProcessScript);
  Tcl_DString* getPreProcessScript();
  Tcl_DString* getPostProcessScript();

  const char*  getProcessOwner();
  int          getStatus();
  const char*  getStatusStr();
  void         setStatus(int ECprocessStatus);

  void         setNextECProcess(ECProcess* NextECProc);
  ECProcess*   getNextECProcess();
};

#endif /* ECProcess_SEEN_ */
