// CVS ID : $Id: ECPulse.cc,v 1.1 2011/07/09 15:15:01 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECPulse
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.1 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 04.07.2011 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#include "ECPulse.h"
#include <cmath>

using namespace std;


// -------------------------------------------------------------------------
//
// Tcl interface


// Command to create a new PULSE object
//

int ECTk_PulseCmd(ClientData clientData, Tcl_Interp *interp, 
		  int objc,Tcl_Obj *const objv[])
{
  ECPulse* pulse = new ECPulse(interp);
  return ECTk_CreateECProcessCmd(interp,clientData,pulse,objc,objv);
}


// -------------------------------------------------------------------------


// -----------------------------------------------------------------------
//
// class ECPulse : pulse train
//


// ---- HANDLEEVENT : define how ECPulse reacts to external events
//

void ECPulse::handleEvent(ECEvent* ev)
{
  ECProcedure::handleEvent(ev);

  // if not Active nothing to do
  if (status!=Active) return;

  // checks if pulse train should be stopped
  if (timeSinceActivated()>=duration)
    {
      terminate();
      ev->setCmdEvent(CmdProcessTerminated,this);
    }
}


// ---- POTENTIAL : Computes E(t)
//                  Implements a pulse train

double ECPulse::potential()
{
  // computes time since last time pulse switched to ON
  double t=timeSinceActivated();
  unsigned long int n=floor(1000*t/(Ton+Toff));
  t=t-n*(Ton+Toff)/1000;

  // computes potential to output
  if (1000*t<=Ton) 
    return high;
  else
    return low;
}


// ---- INFO : returns a Tcl_DString containing relevant info about 
//             the electrochemical procedure

Tcl_DString* ECPulse::info(const char* frontStr)
{
  ECProcedure::info(frontStr);
  addInfo(frontStr,"pulse high value (V) = ",high);
  addInfo(frontStr,"pulse low value (V) = ",low);
  addInfo(frontStr,"pulse-on time (ms) = ",Ton);
  addInfo(frontStr,"pulse-off time (ms) = ",Toff);
  addInfo(frontStr,"pulse train duration (s) = ",duration);
  return infoStr;
}
