// This may look like C code, but it is really -*- C++ -*-
// CVS ID : $Id: ECPulse.h,v 1.1 2011/07/09 15:15:01 wuthrich Exp $
// ---------------------------------------------------------------- 
//                                                                  
// Header file for  class ECPulse
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.1 $)
//
// This software is copyright under the BSD license
//
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 04.07.2011 Creation                         R. Wuthrich
//                                                                  
// ---------------------------------------------------------------

#ifndef ECPULSE_SEEN_
#define ECPULSE_SEEN_

#include <tcl.h>
#include "ECProcedure.h"

using namespace std;


// -----------------------------------------------------------------------
//
// Tcl interface
//

int ECTk_PulseCmd(ClientData clientData, Tcl_Interp *interp, 
	       int objc,Tcl_Obj *const objv[]);


// -----------------------------------------------------------------------
//
// class ECPULSE : cyclic voltammetry
//

class ECPulse : public ECProcedure
{
protected:

  double high;     // Pulse high value
  double low;      // Pulse low value
  double Ton;      // Pulse-On time in ms
  double Toff;     // Pulse-Off in ms
  double duration; // Pulse-train duration in sec
  

public:

  ECPulse(Tcl_Interp* Interp) : 
    ECObj("Pulse train","pulse",Interp),
    ECProcedure("Pulse train","pulse",Interp),
    high(0), low(0), Ton(0), Toff(0), duration(0)
  {
    addOption("-pulseHigh",&high,"returns/sets pulse high value (V)");
    addOption("-pulseLow",&low,"returns/sets pulse low value (V)");
    addOption("-Ton",&Ton,"returns/sets pulse on-time (ms)");
    addOption("-Toff",&Toff,"returns/sets pulse off-time (ms)");
    addOption("-duration",&duration,"returns/sets pulse train duration (sec)");
  }

  virtual void handleEvent(ECEvent* ev);
  virtual double potential();
  virtual Tcl_DString* info(const char* frontStr = "");
};


#endif /* ECPULSE_SEEN_ */
