// CVS ID : $Id: ECStep.cc,v 1.7 2010/12/05 20:14:48 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECStep
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.7 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#include "ECStep.h"

using namespace std;

// -------------------------------------------------------------------------
//
// Tcl interface


// Command to create a new STEP object
//

int ECTk_StepCmd(ClientData clientData, Tcl_Interp *interp, 
		 int objc,Tcl_Obj *const objv[])
{
  ECStep* step = new ECStep(interp);
  return ECTk_CreateECProcessCmd(interp,clientData,step,objc,objv);
}

// -------------------------------------------------------------------------



// -----------------------------------------------------------------------
//
// class ECStep : holds potential at a fixed value 
//                during a fixed time interval
//

// ---- HANDLEEVENT : define how ECStep reacts to external events
//

void ECStep::handleEvent(ECEvent* ev)
{
  ECProcedure::handleEvent(ev);

  char str[80];
  char s[TCL_DOUBLE_SPACE];

  // checks if the procedure was activated
  if ((ev->cmd()==CmdProcedureActivated)&&(verbose==1)&&(status==Active))
    {
      Tcl_PrintDouble(interp,EStep,s);
      sprintf(str,"cons \"\t -- step value %s\"",s); 
      Tcl_Eval(interp,str);
    }

  // checks if experiment is finished
  if ((timeSinceActivated()>=stepDuration)&&(status==Active))
    {
      terminate();
      ev->setCmdEvent(CmdProcessTerminated,this);
    }
}


// ---- POTENTIAL : Computes E(t)
//                  Outputs EStep

double ECStep::potential()
{
  return EStep;
}


// ---- INFO : returns a Tcl_DString containing relevant info about 
//             the electrochemical procedure

Tcl_DString* ECStep::info(const char* frontStr)
{
  ECProcedure::info(frontStr);
  addInfo(frontStr,"step potential (V) = ",EStep);
  addInfo(frontStr,"step duration (s) = ",stepDuration);
  return infoStr;
}
