// CVS ID : $Id: ECTrigger.cc,v 1.9 2011/07/09 15:15:01 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECTrigger
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.9 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// 08.07.2011 Added trigger types              R. Wuthrich
// ---------------------------------------------------------------

#include <tcl.h>
#include <cstring>
#include "ECTrigger.h"

using namespace std;


// -------------------------------------------------------------------------
//
// Tcl interface
//


// Command to create a new trigger object
//

int ECTk_TriggerCmd(ClientData clientData, Tcl_Interp *interp, 
		    int objc,Tcl_Obj *const objv[])
{
  ECTrigger* trig  = new ECTrigger(interp);
  return ECTk_CreateECProcessCmd(interp,clientData,trig,objc,objv);
}

// -------------------------------------------------------------------------


// -------------------------------------------------------------------------
//
// class ECTrigger : a class implementing triggers
//


// ---- DESTRUCTOR
//

ECTrigger::~ECTrigger()
{
  Tcl_DStringFree(triggerScript);
  delete triggerScript;
}


// ---- CMD : process a potential command option (# i of objv[])
//            searches the options table 
//            if a match is found processes the command option and
//            returns the index from the options table
//            if no match returns -1

int ECTrigger::cmd(int &i, int objc,Tcl_Obj *const objv[])
{
  // first executes the command options defined in ECProcess
  int index=ECProcess::cmd(i,objc,objv);

  if (index==getOptionIndex("-always"))
    {
      trigger_type=Trigger_always;
      i++;
    }

  if (index==getOptionIndex("-single"))
    {
      trigger_type=Trigger_single;
      i++;
    }

  return index;
}


// ---- HANDLEEVENT : define how the ECTrigger reacts to external events
//

void ECTrigger::handleEvent(ECEvent* ev)
{
  ECProcess::handleEvent(ev);

  // checks if the trigger has to be activated

   if ((status==Waiting)&&(ev->cmd()==CmdProcessTerminated))
     {
       activate(ev);
     }

  // If the trigger is Active, evaluates the trigger script.
  // Terminates the trigger process if the script returns 0 or STOP

  if (status==Active)
    {
      Tcl_Eval(interp,Tcl_DStringValue(triggerScript)); 
      if ((strcmp(interp->result,"0")==0)||(strcmp(interp->result,"STOP")==0))
	{
	  if (verbose==1)
	    {
	      char str[80];
	      sprintf(str,"cons \"  ==> trig%s released\"",objID); 
	      Tcl_Eval(interp,str);
	    }
	  Tcl_Eval(interp,Tcl_DStringValue(actionScript));
	  if (trigger_type==Trigger_single) terminate();
	}
    }

  // Necessary as otherwise the result of the TriggerScript would
  // remain in the interpreter
  Tcl_ResetResult(interp);
}


// ---- INFO : returns a Tcl_DString containing relevant info about content
// 

Tcl_DString* ECTrigger::info(const char* frontStr)
{
  ECProcess::info(frontStr);
  addInfo(frontStr,"  ",Tcl_DStringValue(triggerScript));
  Tcl_DStringAppend(infoStr,"\nAction:",-1);
  addInfo(frontStr,"  ",Tcl_DStringValue(actionScript));  
  if (trigger_type==Trigger_always)
    addInfo(frontStr,"Trigger type : always","");
  else
    addInfo(frontStr,"Trigger type : single","");
  return infoStr;
}


// ---- SETTRIGGERSCRIPT : sets the variable postProcessScript
//

void ECTrigger::setTriggerScript(Tcl_DString* TriggerScript)
{
  Tcl_DStringFree(triggerScript);
  Tcl_DStringAppend(triggerScript,Tcl_DStringValue(TriggerScript),-1);
}


// ---- GETTRIGGERSCRIPT : returns the variable triggerScript
//

Tcl_DString* ECTrigger::getTriggerScript()
{
  return triggerScript;
}
