// This may look like C code, but it is really -*- C++ -*-
// CVS ID : $Id: ECTrigger.h,v 1.8 2011/07/09 15:15:01 wuthrich Exp $
// ---------------------------------------------------------------- 
//                                                                  
// Header file for  class ECTrigger
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.8 $)
//
// This software is copyright under the BSD license
//
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// 08.07.2011 Added trigger types              R. Wuthrich
// ---------------------------------------------------------------

#ifndef ECTrigger_SEEN_
#define ECTrigger_SEEN_

#include <tcl.h>
#include <stdio.h>
#include "ECProcess.h"

using namespace std;


// ------------------------------------------------------------------------
//
// Trigger type
//

const int
  Trigger_single = 1,
  Trigger_always = 0;


// -----------------------------------------------------------------------
//
// Tcl interface
//

int ECTk_TriggerCmd(ClientData clientData, Tcl_Interp *interp, 
		    int objc,Tcl_Obj *const objv[]);


// -----------------------------------------------------------------------
//
// class ECTrigger : a class implementing triggers
//

class ECTrigger : public ECProcess
{
private:

  int  trigger_type;

protected:

  Tcl_DString* triggerScript;
  Tcl_DString* actionScript;

public:

  ECTrigger(Tcl_Interp *Interp) :
    ECObj("Trigger","trig",Interp),
    ECProcess("Trigger","user","trig",Interp),
    trigger_type(Trigger_single)
  {
    triggerScript = new Tcl_DString;
    Tcl_DStringInit(triggerScript);
    Tcl_DStringAppend(triggerScript,"1",-1);
    actionScript = new Tcl_DString;
    Tcl_DStringInit(actionScript);
    Tcl_DStringAppend(actionScript,"",-1);
    verbose=0;
    addOption("-triggerScript",triggerScript,"returns/sets trigger condition");
    addOption("-action",actionScript,"returns/sets action");
    addOption("-single","trigger is removed after release");
    addOption("-always","trigger is permanent");
  }

  ~ECTrigger();

  virtual int  cmd(int &i,int objc,Tcl_Obj *const objv[]);
  virtual void handleEvent(ECEvent* ev);
  virtual Tcl_DString* info(const char* frontStr = "");

  void setTriggerScript(Tcl_DString* TriggerScript);
  Tcl_DString* getTriggerScript();
};

#endif /* ECTrigger_SEEN_ */
