// CVS ID : $Id: ECUserProc.cc,v 1.8 2010/08/10 00:46:37 wuthrich Exp $
// ---------------------------------------------------------------
//
// Definition of the class ECUserProc
//
// (c) Rolf Wuthrich
//     2010 Concordia University
//
// author:  Rolf Wuthrich
// email:   wuthrich@encs.concordia.ca
// version: v1 ($Revision: 1.8 $)
//
// This software is copyright under the BSD license
//  
// ---------------------------------------------------------------
// history:
// ---------------------------------------------------------------
// Date       Modification                     Author
// ---------------------------------------------------------------
// 31.03.2010 Creation                         R. Wuthrich
// ---------------------------------------------------------------

#include <tcl.h>
#include "ECUserProc.h"

using namespace std;


// -------------------------------------------------------------------------
//
// Tcl interface


// Command to create a new userproc object
//

int ECTk_UserProcCmd(ClientData clientData, Tcl_Interp *interp, 
		     int objc,Tcl_Obj *const objv[])
{
  ECUserProc* uproc  = new ECUserProc(interp);
  return ECTk_CreateECProcessCmd(interp,clientData,uproc,objc,objv);
}


// -------------------------------------------------------------------------



// -------------------------------------------------------------------------
//
// class ECUserProc : allows ueser defined ECPProcedures


// ---- DESTRUCTOR
//

ECUserProc::~ECUserProc()
{
  Tcl_DStringFree(userProcScript);
  delete userProcScript;
}


// ---- HANDLEEVENT : define how ECUserProc has to react to external events
//

void ECUserProc::handleEvent(ECEvent* ev)
{
  ECProcedure::handleEvent(ev);
}


// ---- POTENTIAL : computes the potential E by evaluating the userProcScript
//                  userProcScript has to return the desired potential


double ECUserProc::potential()
{
  double e;
  Tcl_Eval(interp,Tcl_DStringValue(userProcScript));
  Tcl_GetDoubleFromObj(interp,Tcl_GetObjResult(interp),&e);
  return e;
}


// ---- GETUSERPROCSCRIPT : returns userProcScript
//

Tcl_DString* ECUserProc::getUserProcScript()
{
  return userProcScript;
}


// ---- SETUSERPROCSCRIPT : sets userProcScript
//

void ECUserProc::setUserProcScript(Tcl_DString* UserProcScript)
{
  Tcl_DStringFree(userProcScript);
  Tcl_DStringAppend(userProcScript,Tcl_DStringValue(UserProcScript),-1);
}


// ---- INFO : returns a Tcl_DString containing relvant info about 
//             the electrochemical procedure

Tcl_DString* ECUserProc::info(const char* frontStr)
{
  ECProcedure::info(frontStr);
  addInfo(frontStr,"  ",Tcl_DStringValue(userProcScript));
  return infoStr;
}
