# CVS ID : $Id: plot.tcl,v 1.3 2011/03/08 01:33:28 wuthrich Exp $
# ---------------------------------------------------------------
#
# ECTk interface for plotting
#
# (c) Rolf Wuthrich
#     2010 Concordia University
#
# author:  Rolf Wuthrich
# email:   wuthrich@encs.concordia.ca
# version: v1 ($Revision: 1.3 $)
#
# ---------------------------------------------------------------
# history:
# ---------------------------------------------------------------
# Date       Section       Modification             Author
# ---------------------------------------------------------------
# 22.04.10   all           creation                 R. Wuthrich 
#                                                                  
# ---------------------------------------------------------------

global PlotPipe

# ----------------------------------------------------------------------
#
# openPlotPipe - opens a pipline to gnuplot
#
# usage : openPlotPipe
#
# -----------------------------------------------------------------------

proc openPlotPipe {} {
    global PlotPipe
    if [catch {open |gnuplot w} PlotPipe] {
	puts stderr "Cannot open gnuplot: $PlotPipe"
    }
}


# ----------------------------------------------------------------------
#
# closePlotPipe - closes the pipline to gnuplot
#
# usage : closePlotPipe
#
# -----------------------------------------------------------------------

proc closePlotPipe {} {
    global PlotPipe
    close $PlotPipe
    file delete .ectmpfile*
}



# ----------------------------------------------------------------------
#
# plotStyle - defines the plot style
#
# usage : plotStyle style
#
# -----------------------------------------------------------------------

proc plotStyle {style} {
    set ::ectk::plotArgs [format "$style notitle"]
}


# ----------------------------------------------------------------------
#
# plotGnuplot - plots the content of a file
#
# usage : plotGnuplot file
#
# -----------------------------------------------------------------------

proc plotGnuplot {file {args ""}} {
    global PlotPipe

    if {[catch {flush $PlotPipe}] > 0} {
	openPlotPipe
    }

    puts $PlotPipe "plot \"$file\" $args"
    flush $PlotPipe
}



# ----------------------------------------------------------------------
#
# plot - plots the values aquired during a ECProcedure
#
# usage : plot ECprocCmd
#
# -----------------------------------------------------------------------

proc plot {ECprocCmd args} {
    global PlotPipe

    file delete .ectmpfile*
    [lindex $ECprocCmd 0] -save .ectmpfile \
	[lindex $ECprocCmd 1] [lindex $ECprocCmd 2]
    plotGnuplot .ectmpfile
    foreach cmd $args {
	[lindex $cmd 0] -save .ectmpfile.$cmd [lindex $cmd 1] [lindex $cmd 2]
	puts $PlotPipe "replot \".ectmpfile.$cmd\" $::ectk::plotVars $::ectk::plotArgs"
    }
    flush $PlotPipe
}


# -----------------------------------------------------------------------
#
# sets the standard variables
#

plotStyle "with points"
